package solitaire;
import mWaba.*;
import mWaba.gui.*;
import mWaba.gui.control.*;
import mWaba.gui.graphics.*;
import waba.fx.*;

//##################################################################
public class SolitaireBoard extends InteractivePanel {
//##################################################################

int armWidth, boardWidth, board[], piecesLeft;
final static int Illegal = -1, Empty = 0, Occupied = 1;
public final static int PlayMode = 1, ComposeMode = 2;

public int mode = PlayMode;

public static AniImage piece, groove;
{
	autoScrolling = false;
	quickDragging = true;
}
//==================================================================
boolean isOnBoard(int x,int y)
//==================================================================
{
	if (x < 0 || x >= boardWidth) return false;
	if (y < 0 || y >= boardWidth) return false;
	return true;	
}
//==================================================================
int stateOf(int x,int y)
//==================================================================
{
	return board[y*boardWidth+x];
}
//==================================================================
void setState(int x,int y,int state)
//==================================================================
{
	board[y*boardWidth+x] = state;
}

//==================================================================
void makeBoard(int aw)
//==================================================================
{
	armWidth = aw;
	boardWidth = armWidth*armWidth;
	piecesLeft = (boardWidth*5)-1;
	board = new int[boardWidth*boardWidth];
	for (int i = 0; i<boardWidth*boardWidth; i++) board[i] = Illegal;
	for (int i = 0; i<armWidth; i++) 
		for (int j = 0; j<boardWidth; j++)
			setState(j,i+armWidth,Occupied);
	for (int i = 0; i<armWidth; i++) 
		for (int j = 0; j<boardWidth; j++)
			setState(i+armWidth,j,Occupied);
	setState(boardWidth/2,boardWidth/2,Empty);
}

int pieceWidth;
AniImage background;

//==================================================================
Point putImageAt(AniImage what,int x,int y)
//==================================================================
{
	int sp = (pieceWidth-what.getDim().width)/2;
	Point p =  new Point(x*pieceWidth+sp,y*pieceWidth+sp);
	return p;
}
//==================================================================
public void makeBackground()
//==================================================================
{
	int w = pieceWidth*boardWidth;
	Image i = new Image(w,w);
	Graphics g = new Graphics(i);
	mColor.setColor(g,mColor.LightGray);
	g.fillRect(0,0,w,w);
	groove.drawMode = g.DRAW_OR;
	for (int x = 0; x<boardWidth; x++)
		for (int y = 0; y<boardWidth; y++)
			if (stateOf(x,y) != Illegal) {
				Point p = putImageAt(groove,x,y);
				groove.draw(g,p.x,p.y,0);
			}
	g.free();
	background = new AniImage();
	background.setImage(i);
	background.properties |= background.IsNotHot;
	backgroundImage = background.image;//addImage(background);
}
//==================================================================
public void fillPieces()
//==================================================================
{
	for (int x = 0; x<boardWidth; x++)
		for (int y = 0; y<boardWidth; y++)
			if (stateOf(x,y) == Occupied) {
				Point p = putImageAt(groove,x,y);
				AniImage i = (AniImage)piece.getCopy();
				i.move(p.x,p.y);
				i.properties |= i.IsMoveable;
				addImage(i);
			}
}
//==================================================================
public SolitaireBoard(boolean small)
//==================================================================
{
	if (!small) {
		piece = new AniImage("Marble.bmp","MarbleMask.bmp");
		groove = new AniImage("Groove.bmp",null);
	}else {
		piece = new AniImage("SmallMarble.bmp","SmallMarbleMask.bmp");
		groove = new AniImage("SmallGroove.bmp",null);
	}
	pieceWidth = piece.getDim().width;
	checkId = mApp.requestTimer(this,250);
	newBoard();
}

//==================================================================
public void newBoard()
//==================================================================
{
	pauseSnapShots = true;
	images.clear();
	makeBoard(3);	
	makeBackground();
	fillPieces();
	reported = false;
	virtualSize = new Rect(0,0,background.location.height,background.location.height);
	takeSnapShot();
}
//==================================================================
public void clearBoard()
//==================================================================
{
	for (int i = 0; i<board.length; i++)
		if (board[i] != Illegal) board[i] = Empty;
	images.clear();
	//addImage(background);
	piecesLeft = 0;
	takeSnapShot();
	refresh();
}
//==================================================================
protected void calculatePreferredSize()
//==================================================================
{
	virtualSize = new Rect(0,0,0,0);
	preferredSize.width = preferredSize.height = virtualSize.width = virtualSize.height =
	background.getDim().width;
}

Point draggingPiece;
//============================================================
public void draggingStarted(ImageDragContext dc)
//============================================================
{
	showPieces("Dragging started.");
	draggingPiece = null;
	AniImage i = dc.image;
	if (i == null) return;
	images.moveOnTop(i);
	draggingPiece = whichPiece(i.location.x,i.location.y);
	showPieces("Dragging started again.");
}
//==================================================================
public boolean imageDragged(ImageDragContext dc,Point where)
//==================================================================
{
	showPieces("Dragging...");
	return super.imageDragged(dc,where);
}
//==================================================================
void occupy(AniImage i,Point p)
//==================================================================
{
	setState(p.x,p.y,Occupied);
	p = putImageAt(i,p.x,p.y);
	i.move(p.x,p.y);
}
//==================================================================
void showPieces(String message)
//==================================================================
{
	/*
	System.out.println(message);
	for (int i = 0; i<images.size(); i++){
		AniImage im = (AniImage)images.elementAt(i);
		Point at = whichPiece(im.location.x,im.location.y);
		System.out.println(im+" A: "+im.location.x+", "+im.location.y+" P: "+at.x+", "+at.y);
	}
	*/
}
//==================================================================
AniImage pieceAt(Point p)
//==================================================================
{
	for (int i = 0; i<images.size(); i++){
		AniImage im = (AniImage)images.elementAt(i);
		Point at = whichPiece(im.location.x,im.location.y);
		if (at.x == p.x && at.y == p.y) return im;
	}
	return null;
}
//==================================================================
public boolean check(AniImage i,Point p)
//==================================================================
{
	Point s = draggingPiece;
	Point over = new Point(p.x,p.y);
	if (!isOnBoard(p.x,p.y)) return false;
	if (stateOf(p.x,p.y) != Empty) return false;
	int dx = p.x-s.x;
	int dy = p.y-s.y;
	boolean isValid = false;
	if ((dx == 2 || dx == -2) && (dy == 0)) isValid = true;
	if ((dy == 2 || dy == -2) && (dx == 0)) isValid = true;
	if (!isValid) return false;
	over.x = s.x+(dx/2);
	over.y = s.y+(dy/2);
	if (stateOf(over.x,over.y) == Empty) return false;
	occupy(i,p);
	dropImage(i);
	setState(s.x,s.y,Empty);
	setState(over.x,over.y,Empty);
	AniImage jumped = pieceAt(over);
	removeImage(jumped);
	refresh(jumped.location,null);
	piecesLeft--;
	return true;
}
//============================================================
public void draggingStopped(ImageDragContext dc)
//============================================================
{
	showPieces("Stopped dragging.");
	try{
		AniImage i = dc.image;
		if (i == null) return;
		int ix = i.location.x, iy = i.location.y;
		int w = i.getDim().width;
		if (check(i,whichPiece(ix,iy))) return;
		if (check(i,whichPiece(ix+w,iy))) return;
		if (check(i,whichPiece(ix,iy+w))) return;
		if (check(i,whichPiece(ix+w,iy+w))) return;
		occupy(i,draggingPiece);
	}finally{
		showPieces("Erased piece.");
	}
}

//==================================================================
public Point whichPiece(int boardX,int boardY)
//==================================================================
{
	int x = boardX/pieceWidth;
	int y = boardY/pieceWidth;
	return new Point(x,y);
}

boolean reported = false;
int checkId;
//==================================================================
public void changeMode(int newMode)
//==================================================================
{
	if (newMode == mode) return;
	piecesLeft = 0;
	for (int i = 0; i<images.size(); i++){
		AniImage im = (AniImage)images.elementAt(i);
		if (im == background) continue;
		im.properties ^= im.IsMoveable;
		piecesLeft++;
	}
	mode = newMode;
	reported = false;
	takeSnapShot();
}
//==================================================================
public boolean imagePressed(AniImage image,Point where)
//==================================================================
{
	if (mode == PlayMode) return super.imagePressed(image,where);
	else {
		if (image != null) {
			removeImage(image);
			Point pc = whichPiece(image.location.x,image.location.y);
			setState(pc.x,pc.y,Empty);
			refresh(image.location,null);
			piecesLeft--;
		}else{
			Point pc = whichPiece(where.x,where.y);
			if (!isOnBoard(pc.x,pc.y)) return true;
			if (stateOf(pc.x,pc.y) == Illegal) return true;
			AniImage i = (AniImage)piece.getCopy();
			occupy(i,pc);
			addImage(i);
			piecesLeft++;
			i.updated();
			refresh(i.location,null);
		}
	}
	return true;
}
//==================================================================
public void ticked(int id,int howLate)
//==================================================================
{
	if (id != checkId) super.ticked(id,howLate);
	else{
		if (piecesLeft == 1 && !reported && mode == PlayMode) {
			reported = true;
			MessageBox mb = new MessageBox("Winner","Congratulations!\nYou won!",0);
			mb.exec(null,null);
		}
	}
}
//##################################################################
}
//##################################################################

